import React, { useCallback, useEffect, useState } from 'react';
import cn from 'classnames';

import styles from './InputComponent.module.scss';

interface InputComponentProps {
    /** Text displayed in the input */
    value: string;
    /** Label displayed on the input */
    label: string;
    /** Show error flag */
    error: boolean;
    /** onChange listener */
    onChange: (event: React.ChangeEvent<HTMLInputElement>) => void;
}

/**
 *  Component to render a styled input
 */
function InputTextComponent({ value, label, error, onChange }: InputComponentProps) {
    const [labelAsPlaceholder, setLabelAsPlaceholder] = useState(!value);

    useEffect(() => {
        setLabelAsPlaceholder(!value);
    }, [value]);

    const onInputFocus = useCallback(() => {
        setLabelAsPlaceholder(false);
    }, []);

    const onInputBlur = useCallback(() => {
        if (!value) setLabelAsPlaceholder(true);
    }, [value]);

    const handleAutoFill = useCallback((event: React.AnimationEvent) => {
        if (event.animationName === 'onAutoFillStart') {
            setLabelAsPlaceholder(false);
        }
    }, []);

    return (
        <label className={styles.input}>
            <input
                onAnimationStart={handleAutoFill}
                onChange={onChange}
                onFocus={onInputFocus}
                onBlur={onInputBlur}
                className={cn(
                    [styles.inputField],
                    { [styles.inputFieldFocus]: !labelAsPlaceholder },
                    { [styles.error]: error },
                )}
                type="text"
                placeholder=" "
                value={value || ''}
            />
            <span className={styles.inputText}>{label}</span>
        </label>
    );
}

export default InputTextComponent;
