import React, {useRef} from 'react';
import cn from 'classnames';

import styles from './SkillDropdownComponent.module.scss';
import DropArrow from '@assets/icons/ic_down-arrow.svg';
import EmptyIcon from '@assets/icons/ic_empty.svg';
import { resource } from '@assets/text/resources';
import IconSpinner from '@assets/icons/ic_spinner.svg';
import IconWarn from "@assets/icons/ic_some_warn.svg";
import IconCancel from '@assets/icons/ic_cancel.svg';
import IconSearch from '@assets/icons/ic_search.svg';
import { Skill } from '@api/types';
import {useOnClickOutside} from "@src/hooks/useClickOutside";

type SkillDropdownComponentProps = {
    /** Show drop down flag */
    isShow: boolean;
    /** Available array of skills */
    skills: Array<Skill>;
    /** Available array of filtered skills */
    filteredSkills: Array<Skill>
    /** Selected skill id for current transaction */
    selectedSkillId?: string | null;
    /** Event handler for skill changes */
    handleSkillChange: (event: React.MouseEvent) => void;
    setSearchText: ( searchQuery:string )=>void;
    searchText: string
    /** Event handler for hide drop down */
    hideDropdown: (event: MouseEvent | TouchEvent) => void;
    /** Event handler for show toggle */
    toggleShow: () => void;
    /** Is loading flag */
    isLoading: boolean;
    /** Disable flag for drop down */
    disabled: boolean;
    /** Drop down Y anchor */
    dropDownYAnchor?: "bottom"|"top"
    /** Drop down X anchor */
    dropDownXAnchor?: "left"|"right"
};

/**
 *  Component to render a dropdown for display and select skills
 */
const SkillDropdownComponent = ({
    isShow,
    skills,
    filteredSkills,
    selectedSkillId,
    handleSkillChange,
    setSearchText,
    searchText,
    hideDropdown,
    toggleShow,
    isLoading,
    disabled,
    dropDownYAnchor,
    dropDownXAnchor
}: SkillDropdownComponentProps) => {
    const skillListItem = (skill: Skill) => (
        <button
            key={skill.id + skill.name}
            data-value={skill.id}
            onClick={handleSkillChange}
            type="button"
            title={skill.name}
        >
            {skill.name}
        </button>
    );

    const getDivider = () => {
        return (<div className={styles.divider}/>)
    };

    const skillList =
        filteredSkills && filteredSkills.length > 0 ? (
            filteredSkills.map((skill) => skillListItem(skill))
        ) : (
            <div className={styles.empty}>
                <EmptyIcon className={styles.emptyIcon} />
                <span>{resource.title.noSkills}</span>
            </div>
        );

    const loadingWrapper = isLoading ? (
        <div className={styles.loadingContainer}>
            <IconSpinner />
            <span>{resource.title.skillsIsLoading}</span>
        </div>
    ) : (
        skillList
    );
    if (disabled) {
        return (
            <div className={styles.dropdown}>
                <button className={styles.dropButton} type="button">
                    <span className={styles.dropTitleDisabled}>
                        <span>
                            {selectedSkillId
                                ? skills.find((skill) => skill.id === selectedSkillId)?.name
                                : resource.button.selectSkill}
                        </span>
                    </span>
                </button>
            </div>
        );
    }

    const buttonRef = useRef(null);
    useOnClickOutside(buttonRef, hideDropdown)

    return (
        <div className={styles.dropdown} ref={buttonRef}>
            <button
                className={cn([styles.dropButton, styles.dropButtonActive])}
                onClick={toggleShow}
                type="button"
            >
                <span className={styles.dropTitle}>
                    <span>
                        {selectedSkillId
                            ? skills.find((skill) => skill.id === selectedSkillId)?.name
                            : resource.button.selectSkill}
                    </span>
                    <DropArrow className={styles.dropArrow} />
                </span>
            </button>
            {isShow &&
            <div className={cn([styles.dropdownContent, {[styles.dropdownContentTop]: dropDownYAnchor == "top"}, {[styles.dropdownContentLeft]: dropDownXAnchor == "left"}])}>
                <div className={styles.dropdownHeader}>
                    <IconSearch/>
                    <input autoFocus placeholder={resource.title.searchSkill} value={searchText} onChange={input => setSearchText(input.currentTarget.value)}/>
                    <IconCancel className={styles.cancelButton} onClick={()=>setSearchText('')}/>
                </div>
                {getDivider()}
                <div className={styles.dropdownSkills}>

                    {loadingWrapper}
                </div>
                {getDivider()}
                <div className={styles.dropdownFooter}>
                    <IconWarn/> <span>{resource.title.startProcessingWarning}</span>
                </div>
            </div>
            }
        </div>
    );
};

export default SkillDropdownComponent;
