import React, {useCallback, useEffect, useState} from 'react';
import { useSelector } from 'react-redux';
import SkillDropdownComponent from '@commonComponents/SkillDropdownComponent/SkillDropdownComponent';
import { Skill } from '@api/types';
import { skillsIsLoadingSelector } from '@ducks/Skill/selectors/skillsIsLoadingSelector';

type SkillDropdownContainerProps = {
    /** Available array of skills */
    skills: Array<Skill>;

    /** Selected skill id for current transaction */
    selectedSkillId?: string | null;

    /** Event handler for skill changes */
    handleChange: (skillId: string | null) => void;

    /** Disable flag for drop down */
    disabled?: boolean;

    /** Drop down Y anchor */
    dropDownYAnchor?: "bottom"|"top"

    /** Drop down X anchor */
    dropDownXAnchor?: "left"|"right"
};

/**
 *  Component to render a dropdown for display and select skills
 */
const SkillDropdownContainer = ({
    skills,
    selectedSkillId = null,
    handleChange,
    disabled = false,
    dropDownYAnchor = "bottom",
    dropDownXAnchor = "right"
}: SkillDropdownContainerProps) => {
    const [show, setShow] = useState(false);
    const [searchText, setSearchText] = useState("");
    const [filteredSkills, setFilteredSkills] = useState(skills);
    const [filterTimeout, setFilterTimeout] = useState<NodeJS.Timeout>()

    const searchSkill = (searchQuery:string) => {
        if (filterTimeout) clearTimeout(filterTimeout)
        if (!searchQuery) return setFilteredSkills(skills)

        setFilterTimeout(setTimeout(() => {
            setFilteredSkills(skills.filter(
                skill => skill.name.toLowerCase().includes(searchQuery.toLowerCase())
            ))
        }, 500))
    }

    useEffect(()=>{
        searchSkill(searchText)
    }, [searchText])

    const isLoading = useSelector(skillsIsLoadingSelector);
    const toggleShow = useCallback(() => {
        setShow(true);
    }, [setShow]);

    const hideDropdown = useCallback(
        () => {
            setShow(false);
        },
        [setShow],
    );

    const handleSkillChange = useCallback(
        (event: React.MouseEvent) => {
            const skillId = event.currentTarget.getAttribute('data-value');
            handleChange(skillId);
            setShow(false);
        },
        [handleChange, setShow],
    );

    return (
        <SkillDropdownComponent
            isShow={show}
            filteredSkills={filteredSkills}
            skills={skills}
            isLoading={isLoading}
            handleSkillChange={handleSkillChange}
            setSearchText = {setSearchText}
            searchText={searchText}
            selectedSkillId={selectedSkillId}
            hideDropdown={hideDropdown}
            toggleShow={toggleShow}
            disabled={disabled}
            dropDownXAnchor={dropDownXAnchor}
            dropDownYAnchor={dropDownYAnchor}
        />
    );
};

export default SkillDropdownContainer;
