import { ActionPattern,all, call, put, race, take } from 'redux-saga/effects';
import transactionService, {
    GetDocumentsForTransactionResponse,
    GetTransactionInfoResponse
} from '../../../api/transactionService';
import {
    DeleteTransactionActionTypes,
    getTransactionError,
    getTransactionRequest,
    getTransactionStart,
    getTransactionSuccess, setTransactionDownloadSource,
} from '../actions';
import { Type } from '../actionTypes';
import { resource } from '../../../assets/text/resources';
import {ResultFile, TransactionFileType, TransactionStatus} from '../../../api/types';
import { SagaIterator } from 'redux-saga';
import { Action } from 'redux';

export default function* getTransactionSaga({ payload }: ReturnType<typeof getTransactionRequest>): SagaIterator {
    try {
        yield put(getTransactionStart(payload));

        const takeArg: ActionPattern<Action<any>> =
            (action: Action<any>) =>
            {
                const deleteAction = action as DeleteTransactionActionTypes;
                if (deleteAction.type === Type.DELETE_TRANSACTION_SUCCESS)
                {
                    return deleteAction.payload.uuid === payload.uuid;
                }
                else
                    return false;
            }

        const raceResult: { res: any; cancel: any } = yield race({
            res: call(transactionService.getTransactionInfo, payload.transactionId),
            cancel: take(takeArg),
        });
        if (raceResult.cancel) {
            return;
        }
        const response: GetTransactionInfoResponse = raceResult.res;

        const { data } = response;
        const { status } = response.data;

        let result: {
            uuid: string;
            status: TransactionStatus;
            sourceFiles?: TransactionFileType[];
            reviewLink?: string;
            resultFiles?: ResultFile[];
            resultClassNames?: string[];
        };

        if (data.manualReviewLink) {
            // If we found review link in response
            result = {
                uuid: payload.uuid,
                status: TransactionStatus.REVIEW,
                reviewLink: data.manualReviewLink,
            };
        } else {
            result = {
                uuid: payload.uuid,
                status,
            };

            // If we found file for download in response
            if (data.documents?.[0]?.resultFiles && data.documents?.[0]?.resultFiles.length > 0) {
                result.resultFiles = [];
                data.documents.forEach((document, index)=>{
                    let resultFiles: ResultFile[] = [];
                    if(document.resultFiles) {
                        resultFiles = document.resultFiles.map(file => {
                            file.fileIndex = index
                            return file
                        })
                    }
                    result.resultFiles = result.resultFiles?.concat(resultFiles)
                })
            }

            // If we found result class in response
            if (data.documents?.[0]) {
                let classNames: string[] = []
                data.documents?.forEach(document => {
                    if(document.classification?.resultClass) {
                        const {resultClass} = document.classification;
                        classNames = classNames.concat(resultClass.name ? resultClass.name : 'Unknown');
                    }
                })
                classNames = classNames.filter((className, index, classNames) => classNames.indexOf(className) == index)
                result.resultClassNames = classNames;
            }
        }
        // If we need download source file for mobile upload
        if (payload.needLoadSource) {
            yield put(
                setTransactionDownloadSource({
                    uuid: payload.uuid,
                    isDownload: true,
                }),
            );
            const response: GetDocumentsForTransactionResponse = yield call(transactionService.getDocumentsForTransaction, payload.transactionId)
            if (response.data && response.data.length > 0) {
                let sourceFileFetch: any[] = []
                let sourceFileData: { fileId: string, name: string }[] = []
                response.data.forEach(document => {
                    if (document.sourceFiles && document.sourceFiles.length > 0) {
                        sourceFileFetch.push(call(transactionService.downloadDocumentSourceFile, payload.transactionId, document.documentId, document.sourceFiles[0].id))
                        sourceFileData.push({
                            fileId: document.sourceFiles[0].id,
                            name: document.sourceFiles[0].name
                        })
                    }
                })
                const fileArray = yield all(sourceFileFetch)
                const sourceFiles: TransactionFileType[] = fileArray.map((sourceFile: { data: BlobPart; }, index: number) => {
                    const blob = [
                        new Blob([sourceFile.data], {
                            type: 'image/jpg'
                        }),
                        ' Same way as you do with blob',
                        new Uint16Array([33])
                    ];
                    return {
                        fileId: sourceFileData[index].fileId,
                        name: sourceFileData[index].name,
                        file: new File(blob, "Test.jpg", {
                            type: 'image/jpg',
                        } )
                    }
                })
                result.sourceFiles = sourceFiles
            }
            yield put(
                setTransactionDownloadSource({
                    uuid: payload.uuid,
                    isDownload: false,
                }),
            );
        }

        yield put(getTransactionSuccess(result));
    } catch (error) {
        console.error(resource.error.getTransactionFailed);
        console.error(error);
        yield put(getTransactionError({ errorText: resource.error.getTransactionFailed }));
    }
}
