import React, {useCallback, useEffect, useRef, useState} from 'react';
import styles from './CardComponent.module.scss';
import StatusIconComponent from '@commonComponents/StatusIconComponent/StatusIconComponent';
import StatusTextComponent from '@commonComponents/StatusTextComponent/StatusTextComponent';
import PreviewComponent from '../PreviewComponent/PreviewComponent';
import ReviewResultComponent from '@commonComponents/ReviewResultComponent/ReviewResultComponent';
import SkillDropdownContainer from '@commonContainers/SkillDropDownContainer/SkillDropDownContainer';
import {resource} from '@assets/text/resources';
import FilenameComponent from '@commonComponents/FileNameComponent/FilenameComponent';
import {ResultFile, Skill, Transaction, TransactionStatus} from '@api/types';
import DownloadDropDownContainer from '@src/common/containers/DownloadDropDownContainer/DownloadDropDownContainer';
import MobileUploadComponent from "@commonComponents/MobileUploadComponent/MobileUploadComponent";
import {isMobile} from "react-device-detect";
import ReopenQRCodeButtonComponent from "@commonComponents/ReopenQRCodeButtonComponent/ReopenQRCodeButtonComponent";
import FileDropDownContainer from "@commonContainers/FileDropDownContainer/FileDropDownContainer";
import DeleteIcon from "@assets/icons/ic_delete.svg";
import TransactionBrowseButtonContainer
    from "@commonContainers/TransactionBrowseButtonContainer/TransactionBrowseButtonContainer";
import DeleteModalComponent from "@commonComponents/DeleteModalComponent/DeleteModalComponent";
import ClassDropDownContainer from "@commonContainers/ClassDropDownContainer/ClassDropDownContainer";

type CardComponentProps = {
    /** Transaction object */
    transaction: Transaction;
    /** Skill array */
    skills: Array<Skill>;
    /** Event handler for updating skill dropdown */
    handleChange: (transaction: Transaction, skillId: string | null) => void;
    /** Event handler for delete transaction */
    handleDelete: (transaction: Transaction) => void;
    /** Event handler for open QR Code for transaction */
    handleOpen: (transactionId: string) => void;
    /** Event handler for download result of transaction */
    handleDownload: (transaction: Transaction, fileId: string) => void;
    /** Event handler for delete result of transaction */
    handleDeleteFile: (transaction: Transaction, fileId: string) => void;
};

/**
 *  Component to render transaction card
 */
function CardComponent({ transaction, skills, handleChange, handleDelete, handleDeleteFile, handleDownload, handleOpen }: CardComponentProps) {
    const [isDeleteModalOpen, setIsDeleteModalOpen] = useState(false);
    const [dropDownYAnchor, setDropDownYAnchor] = useState<"bottom"|"top">("bottom");
    const [dropDownXAnchor, setDropDownXAnchor] = useState<"left"|"right">("right");

    const ref = useRef<HTMLDivElement>(null)

    const handleDeleteModalToggle = useCallback(() => {
        setIsDeleteModalOpen(prevState => !prevState)
    }, [setIsDeleteModalOpen]);

    const handleSkillChange = useCallback((skillId) => {
        handleChange(transaction, skillId);
    }, [transaction, handleChange]);

    const handleDeleteTransactionClick = useCallback(() => {
        handleDelete(transaction);
    }, [transaction, handleDelete]);

    const handleDeleteFileClick = useCallback((fileId: string) => {
        handleDeleteFile(transaction, fileId);
    }, [transaction, handleDeleteFile]);

    const handleOpenQRClick = useCallback(() => {
        if(transaction.transactionId) {
            handleOpen(transaction.transactionId);
        }
    }, [transaction, handleOpen]);

    const updateDropDown = useCallback(() => {
        if(ref.current) {
            if(document.documentElement.scrollHeight - (ref.current.getBoundingClientRect().bottom + window.scrollY) < 256) {
                setDropDownYAnchor("top");
            } else { setDropDownYAnchor("bottom") }
            if(document.documentElement.scrollWidth - (ref.current.getBoundingClientRect().right + window.scrollX) < 128) {
                setDropDownXAnchor("left");
            } else { setDropDownXAnchor("right") }
        }
    }, [ref.current]);

    const handleDownloadClick = (file: ResultFile) => {
        handleDownload(transaction, file.fileId);
    };

    useEffect(()=>{
        window.addEventListener("resize", updateDropDown)
        return () => window.removeEventListener("resize", updateDropDown);
    },[])

    return (
        <div className={styles.card} ref={ref} onMouseEnter={updateDropDown}>
            <header className={styles.statusSection}>
                <StatusIconComponent status={transaction.status} />
                <StatusTextComponent status={transaction.status} isMobile={transaction.isMobile} />
            </header>

            <section className={styles.fileNameSection}>
                {transaction.transactionFiles.length > 0 && !transaction.isMobile? (
                    transaction.transactionFiles.length > 1 ?
                        <FileDropDownContainer files={transaction.transactionFiles} handleDelete={handleDeleteFileClick}/> :
                        <FilenameComponent filename={transaction.transactionFiles[0].file.name}/>
                ) : null }
                {transaction.isMobile && transaction.transactionFiles.length > 0? (
                    <FilenameComponent filename={resource.title.documents + transaction.transactionFiles.length}/>
                ): null }
                { !transaction.isMobile? <TransactionBrowseButtonContainer transactionId={transaction.uuid}/>: null }
            </section>

            <section className={styles.previewImageSection}>
                <PreviewComponent transaction={transaction} />
            </section>

            <section className={styles.dropdownSection}>
                <SkillDropdownContainer
                    skills={skills}
                    selectedSkillId={transaction.skillId}
                    handleChange={handleSkillChange}
                    disabled={!!(transaction.isMobile && transaction.transactionId)}
                    dropDownYAnchor={dropDownYAnchor}
                    dropDownXAnchor={dropDownXAnchor}
                />
            </section>

            <section className={styles.resultClassSection}>
                {transaction.resultClassNames && transaction.resultClassNames.length>0 && transaction.status === TransactionStatus.PROCESSED ? (
                    <>
                        {transaction.resultClassNames?.length > 1 ?
                            <ClassDropDownContainer classes={transaction.resultClassNames}/> :
                            <span title={transaction.resultClassNames[0]}>{transaction.resultClassNames[0]}</span>
                        }
                    </>
                ) : null}
            </section>

            {transaction.resultFiles ?
                <section className={styles.footerSelectorSection}>
                    <span className={styles.resultSelector}>
                        <DownloadDropDownContainer files={transaction.resultFiles}
                                                   handleDownload={handleDownloadClick}
                                                   isTop
                        />
                    </span>
                    <span />
                    {transaction.deleting ? (
                        <div className={styles.deletingIcon}>
                            <div />
                        </div>
                    ) : (
                        <div className={styles.deleteIcon} onClick={handleDeleteModalToggle}>
                            <DeleteIcon/>
                        </div>
                    )}
                </section>:
                <section className={styles.footerSection}>
                    {transaction.reviewLink && transaction.status === TransactionStatus.REVIEW ? (
                        <ReviewResultComponent
                            link={transaction.reviewLink}
                            className={styles.review}
                            text={resource.button.review}
                        />
                    ) : null}
                    {(isMobile && transaction.isMobile && transaction.status === TransactionStatus.NEW && transaction.mobileUrl) ? (
                        <MobileUploadComponent
                            text={resource.button.scanDoc}
                            link={transaction.mobileUrl}
                            className={styles.review}/>
                    ) : null}
                    {!isMobile && transaction.isMobile && transaction.status === TransactionStatus.NEW ? (
                        <ReopenQRCodeButtonComponent
                            text={resource.button.mobileUpload}
                            handleOpen={handleOpenQRClick}
                            className={styles.review}/>
                    ) : null}
                    <span />
                    {transaction.deleting ? (
                        <div className={styles.deletingIcon}>
                            <div />
                        </div>
                    ) : (
                        <div className={styles.deleteIcon} onClick={handleDeleteModalToggle}>
                            <DeleteIcon/>
                        </div>
                    )}
                </section>
            }
            {isDeleteModalOpen ? <DeleteModalComponent deleteHandler={handleDeleteTransactionClick}
                                                       closeHandler={handleDeleteModalToggle}
                                                       titleText={resource.title.deleteTransaction}
                                                       subtitleText={resource.title.deleteTransactionSub}/> : null}
        </div>
    );
}

export default CardComponent;
