import React, {useCallback, useRef, useState} from 'react';
import styles from './TableRowComponent.module.scss';
import StatusTextComponent from '@commonComponents/StatusTextComponent/StatusTextComponent';
import StatusIconComponent from '@commonComponents/StatusIconComponent/StatusIconComponent';
import FilenameComponent from '@commonComponents/FileNameComponent/FilenameComponent';
import ReviewResultComponent from '@commonComponents/ReviewResultComponent/ReviewResultComponent';

import DeleteIcon from '@assets/icons/ic_delete.svg';
import DeletingIcon from '@assets/icons/ic_deleting.svg';
import SkillDropdownContainer from '@commonContainers/SkillDropDownContainer/SkillDropDownContainer';
import { resource } from '@assets/text/resources';
import {ResultFile, Skill, Transaction, TransactionStatus} from '@api/types';
import DownloadDropDownContainer from "@commonContainers/DownloadDropDownContainer/DownloadDropDownContainer";
import {isMobile} from "react-device-detect";
import ReopenQRCodeButtonComponent from "@commonComponents/ReopenQRCodeButtonComponent/ReopenQRCodeButtonComponent";
import FileDropDownContainer from "@commonContainers/FileDropDownContainer/FileDropDownContainer";
import TransactionBrowseButtonContainer
    from "@commonContainers/TransactionBrowseButtonContainer/TransactionBrowseButtonContainer";
import DeleteModalComponent from "@commonComponents/DeleteModalComponent/DeleteModalComponent";
import ClassDropDownContainer from "@commonContainers/ClassDropDownContainer/ClassDropDownContainer";

interface ITableRowComponentProperties {
    /**  Transaction object */
    transaction: Transaction;
    /** Array of skills */
    skills: Array<Skill>;
    /** Event handler for updating skill dropdown */
    handleChange: (transaction: Transaction, skillId: string | null) => void;
    /** Event handler for delete transaction */
    handleDelete: (transaction: Transaction) => void;
    /** Event handler for download result of transaction */
    handleDownload: (transaction: Transaction, fileType: string) => void;
    /** Event handler for open QR Code for transaction */
    handleOpen: (transactionId: string) => void;
    /** Event handler for delete result of transaction */
    handleDeleteFile: (transaction: Transaction, fileId: string) => void;
}

/**
 *  Component to render transaction table row
 */
function TableRowComponent({
    transaction,
    skills,
    handleChange,
    handleDelete,
    handleDeleteFile,
    handleDownload,
    handleOpen
}: ITableRowComponentProperties) {
    const [isDeleteModalOpen, setIsDeleteModalOpen] = useState(false);
    const [dropDownYAnchor, setDropDownYAnchor] = useState<"bottom"|"top">("bottom");

    const ref = useRef<HTMLDivElement>(null)

    const handleDeleteModalToggle = useCallback(() => {
        setIsDeleteModalOpen(prevState => !prevState)
    }, [setIsDeleteModalOpen]);

    const handleSkillChange = useCallback(
        (skillId) => {
            handleChange(transaction, skillId);
        },
        [handleChange, transaction],
    );

    const handleOpenQRClick = useCallback(() => {
        if(transaction.transactionId) {
            handleOpen(transaction.transactionId);
        }
    }, [transaction, handleOpen]);

    const handleDeleteTransactionClick = useCallback(() => {
        handleDelete(transaction);
    }, [handleDelete, transaction]);

    const handleDeleteFileClick = useCallback((fileId: string) => {
        handleDeleteFile(transaction, fileId);
    }, [transaction, handleDeleteFile]);

    const updateDropDown = useCallback(() => {
        if(ref.current) {
            if(document.documentElement.scrollHeight - (ref.current.getBoundingClientRect().bottom + window.scrollY) < 256) {
                setDropDownYAnchor("top");
            } else { setDropDownYAnchor("bottom") }
        }
    }, [ref.current]);

    const handleDownloadClick = (file: ResultFile) => {
        handleDownload(transaction, file.fileId);
    };


    return (
        <div className={styles.container} ref={ref} onMouseEnter={updateDropDown}>
            <span>
                <StatusIconComponent status={transaction.status} />
            </span>
            <span>
                <StatusTextComponent status={transaction.status} isMobile={transaction.isMobile}/>
            </span>
            <span>
                {transaction.transactionFiles.length>0 && transaction.transactionFiles[0].file ? (
                    transaction.transactionFiles.length > 1 ?
                        <FileDropDownContainer files={transaction.transactionFiles} handleDelete={handleDeleteFileClick}/> :
                        <FilenameComponent filename={transaction.transactionFiles[0].file.name}/>
                ) : transaction.mobileSourceFileIds ? (
                    <FilenameComponent filename={resource.title.documents + (transaction.mobileSourceFileIds.length + 1)} />
                ) : null}
                { !transaction.isMobile? <TransactionBrowseButtonContainer transactionId={transaction.uuid}/>: null }
            </span>
            <span>
                <SkillDropdownContainer
                    skills={skills}
                    selectedSkillId={transaction.skillId}
                    handleChange={handleSkillChange}
                    dropDownYAnchor={dropDownYAnchor}
                />
            </span>
            <span />
            <span id="Result">
                {transaction.resultClassNames && transaction.resultClassNames.length>0 && transaction.status === TransactionStatus.PROCESSED ? (
                    transaction.resultClassNames?.length > 1 ?
                            <ClassDropDownContainer classes={transaction.resultClassNames}/> :
                            <span title={transaction.resultClassNames[0]}>{transaction.resultClassNames[0]}</span>

                ) : null}
                {transaction.reviewLink && transaction.status === TransactionStatus.REVIEW ? (
                    <ReviewResultComponent
                        text={resource.button.reviewResult}
                        link={transaction.reviewLink}
                        className={styles.review}
                    />
                ) : null}
                {!isMobile && transaction.isMobile && transaction.status === TransactionStatus.NEW? (
                    <ReopenQRCodeButtonComponent
                        text={resource.button.mobileUpload}
                        handleOpen={handleOpenQRClick}
                        className={styles.review}/>
                ) : null}
                {transaction.resultFiles && transaction.resultFiles.length>0 && transaction.status === TransactionStatus.PROCESSED ? (
                    <span className={styles.resultSelector}>
                        <DownloadDropDownContainer files={transaction.resultFiles}
                                                   handleDownload={handleDownloadClick}/>
                    </span>
                ) : null}
            </span>
            <span id="Delete" className={styles.deleteIcon}>
                {transaction.deleting ? (
                    <DeletingIcon className={styles.deletingIcon} />
                ) : (
                    <DeleteIcon className={styles.deleteIcon} onClick={handleDeleteModalToggle} />
                )}
            </span>
            {isDeleteModalOpen ? <DeleteModalComponent deleteHandler={handleDeleteTransactionClick}
                                                       closeHandler={handleDeleteModalToggle}
                                                       titleText={resource.title.deleteTransaction}
                                                       subtitleText={resource.title.deleteTransactionSub}/> : null}
        </div>
    );
}

export default TableRowComponent;
